@@ -591,6 +591,7 @@ typedef struct {
 	gint	  	  unknown_depth; /* handle recursive unknown tags */
 	gboolean	  from_unknown_handler;
 	gboolean          debug_parsing;
+	gboolean          silent_unknowns;
 
 	GSList	 	 *extension_stack; /* stack of GsfXMLInExtension */
 } GsfXMLInInternal;
@@ -699,7 +700,7 @@ lookup_child (GsfXMLInInternal *state, int default_ns_id,
 }
 
 static void
-gsf_xml_dump_state (GsfXMLInInternal *state)
+gsf_xml_in_dump_state (GsfXMLInInternal *state)
 {
 	GSList *ptr = state->pub.node_stack = g_slist_reverse (state->pub.node_stack);
 	if (ptr != NULL)	/* skip toplevel catch all */
@@ -725,7 +726,7 @@ gsf_xml_in_start_element (GsfXMLInInternal *state, xmlChar const *name, xmlChar
 	GSList *ptr;
 	char const *tmp;
 	int i;
-	gboolean complain = TRUE;
+	gboolean complain = !state->silent_unknowns;
 
 	/* Scan for namespace declarations.  Yes it is ugly to have the api
 	 * flag that its children can declare namespaces. However, given that a
@@ -829,7 +830,7 @@ gsf_xml_in_start_element (GsfXMLInInternal *state, xmlChar const *name, xmlChar
 		return;
 
 	g_printerr ("Unexpected element '%s' in state : \n\t", name);
-	gsf_xml_dump_state (state);
+	gsf_xml_in_dump_state (state);
 }
 
 static void
@@ -957,6 +958,7 @@ gsf_xml_in_start_document (GsfXMLInInternal *state)
 	state->contents_stack	= NULL;
 	state->from_unknown_handler = FALSE;
 	state->debug_parsing = gsf_debug_flag ("xml-parsing");
+	state->silent_unknowns = FALSE;
 }
 
 static void
@@ -1180,23 +1182,40 @@ gsf_xml_in_doc_add_nodes (GsfXMLInDoc *doc,
 			  GsfXMLInNode const *nodes)
 {
 	GsfXMLInNode const *e_node;
-	GsfXMLInNodeInternal *tmp, *node;
 
 	g_return_if_fail (doc != NULL);
 	g_return_if_fail (nodes != NULL);
 
 	for (e_node = nodes; e_node->id != NULL ; e_node++ ) {
-		node = g_hash_table_lookup (doc->symbols, e_node->id);
+		GsfXMLInNodeInternal *tmp, *node =
+			g_hash_table_lookup (doc->symbols, e_node->id);
 		if (node != NULL) {
-			/* if its empty then this is just a recusion */
-			if (e_node->start != NULL || e_node->end != NULL ||
-			    e_node->has_content != GSF_XML_NO_CONTENT ||
-			    e_node->user_data.v_int != 0) {
-				g_warning ("ID '%s' has already been registered.\n"
-					   "The additional decls should not specify start,end,content,data", e_node->id);
+			/*
+			 * We use the repeat of a node name to attach an
+			 * entire subtree in another place too.  The second
+			 * node should either be empty (old method) or
+			 * use GSF_XML_2ND (new, as-of 1.14.33).
+			 */
+			if (e_node->has_content == GSF_XML_2ND) {
+				/* Nothing, but e_node contents ignored. */
+			} else if (e_node->start != NULL || e_node->end != NULL ||
+				 e_node->has_content != GSF_XML_NO_CONTENT ||
+				 e_node->user_data.v_int != 0) {
+				g_warning ("ID '%s' has already been registered.",
+					e_node->id);
 				continue;
 			}
+
+			if (strcmp (e_node->parent_id, node->pub.parent_id) == 0)
+				g_warning ("Duplicate node %s (under %s)", e_node->id, e_node->parent_id);
 		} else {
+			if (e_node->has_content == GSF_XML_2ND) {
+				g_warning ("ID '%s' is declared 2nd, but is missing.",
+					e_node->id);
+				/* Hence e_node contents ignored. */
+				continue;
+			}			
+
 			node = g_new0 (GsfXMLInNodeInternal, 1);
 			node->pub = *e_node;
 			/* WARNING VILE HACK :
@@ -1360,6 +1379,24 @@ gsf_xml_in_get_input (GsfXMLIn const *xin)
 	return state->input;
 }
 
+/**
+ * gsf_xml_in_set_silent_unknowns:
+ * @xin: #GsfXMLIn
+ * @silent: whether to be silent about unknown tags
+ *
+ * (New in 1.14.33)
+ *
+ * This provides a means to silently ignore unknown tags in contexts where
+ * they are expected.
+ **/
+void
+gsf_xml_in_set_silent_unknowns (GsfXMLIn *xin, gboolean silent)
+{
+	GsfXMLInInternal *state = (GsfXMLInInternal *)xin;
+	state->silent_unknowns = silent;
+}
+
+
 /**
  * gsf_xml_in_check_ns:
  * @xin: #GsfXMLIn
@@ -1434,6 +1471,7 @@ enum {
 typedef enum {
 	GSF_XML_OUT_NOCONTENT,
 	GSF_XML_OUT_CHILD,
+	GSF_XML_OUT_CHILD_PRETTY,
 	GSF_XML_OUT_CONTENT
 } GsfXMLOutState;
 
@@ -1577,7 +1615,7 @@ gsf_xml_out_new (GsfOutput *output)
  * @xout: #GsfXMLOut
  * @type: the document type declaration
  *
- * Store some optional some &lt;!DOCTYPE .. &gt; content
+ * Store some optional &lt;!DOCTYPE .. &gt; content
  **/
 void
 gsf_xml_out_set_doc_type (GsfXMLOut *xout, char const *type)
@@ -1587,6 +1625,44 @@ gsf_xml_out_set_doc_type (GsfXMLOut *xout, char const *type)
 	priv->doc_type = g_strdup (type);
 }
 
+/**
+ * gsf_xml_out_get_pretty_print:
+ * @xout: #GsfXMLOut
+ *
+ * Returns: the current state of the pretty-print flag.  Note, that
+ * gsf_xml_out_set_pretty_print will return the same value.
+ **/
+gboolean
+gsf_xml_out_get_pretty_print (GsfXMLOut *xout)
+{
+	g_return_val_if_fail (GSF_IS_XML_OUT (xout), TRUE);
+	return xout->priv->pretty_print;
+}
+
+/**
+ * gsf_xml_out_set_pretty_print:
+ * @xout: #GsfXMLOut
+ * @pp: new state of pretty-print flag.
+ *
+ * Returns: the previous state of the pretty-print flag.
+ **/
+gboolean
+gsf_xml_out_set_pretty_print (GsfXMLOut *xout, gboolean pp)
+{
+	gboolean res;
+
+	g_return_val_if_fail (GSF_IS_XML_OUT (xout), TRUE);
+
+	pp = !!pp;
+
+	res = xout->priv->pretty_print;
+	if (pp != res) {
+		xout->priv->pretty_print = pp;
+		g_object_notify (G_OBJECT (xout), "pretty-print");
+	}
+	return res;
+}
+
 static inline void
 gsf_xml_out_indent (GsfXMLOut *xout)
 {
@@ -1667,23 +1743,24 @@ gsf_xml_out_end_element (GsfXMLOut *xout)
 	priv->stack = g_slist_remove (priv->stack, id);
 	priv->indent--;
 	switch (priv->state) {
-	case GSF_XML_OUT_NOCONTENT :
+	case GSF_XML_OUT_NOCONTENT:
 		if (priv->pretty_print)
 			gsf_output_write (xout->output, 3, "/>\n");
 		else
 			gsf_output_write (xout->output, 2, "/>");
 		break;
 
-	case GSF_XML_OUT_CHILD :
+	case GSF_XML_OUT_CHILD_PRETTY:
 		gsf_xml_out_indent (xout);
 	/* fall through */
-	case GSF_XML_OUT_CONTENT :
+	case GSF_XML_OUT_CHILD:
+	case GSF_XML_OUT_CONTENT:
 		if (priv->pretty_print)
 			gsf_output_printf (xout->output, "</%s>\n", id);
 		else
 			gsf_output_printf (xout->output, "</%s>", id);
 	}
-	priv->state = GSF_XML_OUT_CHILD;
+	priv->state = priv->pretty_print ? GSF_XML_OUT_CHILD_PRETTY : GSF_XML_OUT_CHILD;
 	return id;
 }
 
@@ -1923,14 +2000,15 @@ void
 gsf_xml_out_add_float (GsfXMLOut *xout, char const *id,
 		       double val, int precision)
 {
-	char format_str[4 * sizeof (int) + 10];
-	char buf[G_ASCII_DTOSTR_BUF_SIZE + DBL_DIG];
+	char buf[G_ASCII_DTOSTR_BUF_SIZE + DBL_DIG + 17];
 
-	if (precision < 0 || precision > DBL_DIG)
-		precision = DBL_DIG;
-
-	sprintf (format_str, "%%.%dg", precision);
-	g_ascii_formatd (buf, sizeof (buf), format_str, val);
+	if (precision < 0 || precision > 17) {
+		g_ascii_dtostr (buf, sizeof (buf), val);
+	} else {
+		char format_str[4 * sizeof (int) + 10];
+		sprintf (format_str, "%%.%dg", precision);
+		g_ascii_formatd (buf, sizeof (buf), format_str, val);
+	}
 	gsf_xml_out_add_cstr_unchecked (xout, id, buf);
 }
 
