@@ -544,7 +544,6 @@ static void
 gsf_outfile_open_pkg_finalize (GObject *obj)
 {
 	GsfOutfileOpenPkg *open_pkg = GSF_OUTFILE_OPEN_PKG (obj);
-	GSList *ptr;
 
 	if (open_pkg->sink != NULL) {
 		g_object_unref (open_pkg->sink);
@@ -553,9 +552,9 @@ gsf_outfile_open_pkg_finalize (GObject *obj)
 	g_free (open_pkg->content_type);
 	open_pkg->content_type = NULL;
 
-	for (ptr = open_pkg->children ; ptr != NULL ; ptr = ptr->next)
-		g_object_unref (ptr->data);
-	g_slist_free (open_pkg->children);
+	g_slist_free_full (open_pkg->children, g_object_unref);
+	open_pkg->children = NULL;
+
 	parent_class->finalize (obj);
 }
 
@@ -592,8 +591,11 @@ gsf_outfile_open_pkg_new_child (GsfOutfile *parent,
 	gsf_outfile_open_pkg_set_sink (child, sink);
 	g_object_unref (sink);
 
-	open_pkg->children = g_slist_prepend (open_pkg->children, child);
-	g_object_ref (child);
+	/*
+	 * Holding a ref here is not ideal.  It means we won't release any of the
+	 * children until the package is closed.
+	 */
+	open_pkg->children = g_slist_prepend (open_pkg->children, g_object_ref (child));
 
 	return GSF_OUTPUT (child);
 }
@@ -607,16 +609,15 @@ gsf_open_pkg_write_content_default (GsfXMLOut *xml, char const *ext, char const
 	gsf_xml_out_end_element (xml); /* </Default> */
 }
 static void
-gsf_open_pkg_write_content_override (GsfOutfileOpenPkg const *open_pkg,
+gsf_open_pkg_write_content_override (GsfOutfileOpenPkg *open_pkg,
 				     char const *base,
 				     GsfXMLOut *xml)
 {
-	GsfOutfileOpenPkg const *child;
-	char   *path;
 	GSList *ptr;
 
 	for (ptr = open_pkg->children ; ptr != NULL ; ptr = ptr->next) {
-		child = ptr->data;
+		GsfOutfileOpenPkg *child = ptr->data;
+		char *path;
 		if (child->is_dir) {
 			path = g_strconcat (base, gsf_output_name (GSF_OUTPUT (child)), "/", NULL);
 			gsf_open_pkg_write_content_override (child, path, xml);
@@ -632,6 +633,10 @@ gsf_open_pkg_write_content_override (GsfOutfileOpenPkg const *open_pkg,
 		}
 		g_free (path);
 	}
+
+	/* Dispose of children here to break link cycles.  */
+	g_slist_free_full (open_pkg->children, g_object_unref);
+	open_pkg->children = NULL;
 }
 
 static gboolean
@@ -660,6 +665,8 @@ gsf_outfile_open_pkg_close (GsfOutput *output)
 			"application/vnd.openxmlformats-officedocument.spreadsheetml.printerSettings");
 		gsf_open_pkg_write_content_default (xml, "xml",
 			"application/xml");
+		gsf_open_pkg_write_content_default (xml, "vml",
+			"application/vnd.openxmlformats-officedocument.vmlDrawing");
 		gsf_open_pkg_write_content_override (open_pkg, "/", xml);
 		gsf_xml_out_end_element (xml); /* </Types> */
 		g_object_unref (xml);
@@ -718,6 +725,7 @@ gsf_outfile_open_pkg_close (GsfOutput *output)
 	/* close the container */
 	if (NULL == gsf_output_name (output))
 		return gsf_output_close (open_pkg->sink);
+
 	return res;
 }
 
@@ -807,8 +815,6 @@ void
 gsf_outfile_open_pkg_set_content_type (GsfOutfileOpenPkg *open_pkg,
 				       char const *content_type)
 {
-	g_return_if_fail (content_type != NULL);
-
 	if (open_pkg->content_type != content_type) {
 		g_free (open_pkg->content_type);
 		open_pkg->content_type = g_strdup (content_type);
